﻿/* 
 * © Copyright 2013 by ScaleOut Software, Inc.
 *
 * LICENSE AND DISCLAIMER
 * ----------------------
 * This material contains sample programming source code ("Sample Code").
 * ScaleOut Software, Inc. (SSI) grants you a nonexclusive license to compile, 
 * link, run, display, reproduce, and prepare derivative works of 
 * this Sample Code.  The Sample Code has not been thoroughly
 * tested under all conditions.  SSI, therefore, does not guarantee
 * or imply its reliability, serviceability, or function. SSI
 * provides no support services for the Sample Code.
 *
 * All Sample Code contained herein is provided to you "AS IS" without
 * any warranties of any kind. THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGMENT ARE EXPRESSLY
 * DISCLAIMED.  SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED
 * WARRANTIES, SO THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO 
 * EVENT WILL SSI BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, 
 * SPECIAL OR OTHER CONSEQUENTIAL DAMAGES FOR ANY USE OF THE SAMPLE CODE
 * INCLUDING, WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS 
 * INTERRUPTION, LOSS OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION
 * HANDLING SYSTEM OR OTHERWISE, EVEN IF WE ARE EXPRESSLY ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGES.
 */

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;

namespace Common
{
    /// <summary>
    /// Dictionary of Securities with their latest prices. Also contains additional parameter(s)
    /// that are useful during a NamedCache.Invoke operation. This dictionary is used by the 
    /// MarketSimulator's ExchangeSimulator class to store prices of securities, and it is also
    /// passed as an argument to the NamedCache.Invoke distributed operation, giving all worker
    /// nodes an instant snapshot of the latest exchange prices when updating and evaluating 
    /// strategies.
    /// </summary>
    [Serializable]
    public class SecurityDictionary : KeyedCollection<string, Security>
    {
        /// <summary>
        /// Gets/sets the max % deviation that a position is allowed to have in a strategy 
        /// before an alert is tripped on a strategy.
        /// </summary>
        /// <remarks>
        /// While this property doesn't have anything to do with the collection of securities
        /// per se, it's very useful to have this setting on the SecurityDictionary because instances
        /// of this collection are passed as arguments to distributed NamedCache.Invoke operations. As
        /// such, it's a good vehicle for passing invoke-level settings across the entire farm.
        /// </remarks>
        public int AlertThresholdPercent { get; set; }

        /// <summary>
        /// Required infrastructure method for KeyedCollection implementations.
        /// Extracts the key from the provided element.
        /// </summary>
        protected override string GetKeyForItem(Security item)
        {
            if (item == null) throw new ArgumentNullException("item");

            return item.Ticker;
        }

        /// <summary>
        /// Returns the keys (ticker symbols) for all items in the dictionary.
        /// </summary>
        public ICollection<string> Keys
        {
            get
            {
                // keys are available in the protected, internal dictionary of the base class.
                return this.Dictionary.Keys;
            }
        }

        /// <summary>
        /// Creates a deep copy of the collection.
        /// </summary>
        /// <returns></returns>
        public SecurityDictionary DeepCopy()
        {
            SecurityDictionary deepCopy = new SecurityDictionary();
            foreach (Security item in this)
            {
                deepCopy.Add(item.DeepCopy());
            
            }
            deepCopy.AlertThresholdPercent = this.AlertThresholdPercent;

            return deepCopy;
        }
    }


    /// <summary>
    /// Security class containing a security's latest price.
    /// </summary>
    [Serializable]
    public class Security
    {
        public Security(string ticker, decimal price, DateTime timestamp)
        {
            Ticker = ticker;
            Price = price;
            Timestamp = timestamp;
        }

        public string Ticker { get; set; }
        public decimal Price { get; set; }
        public DateTime Timestamp { get; set; }

        public Security DeepCopy()
        {
            return new Security(Ticker, Price, Timestamp);
        }
    }
}
