﻿/* 
 * © Copyright 2013 by ScaleOut Software, Inc.
 *
 * LICENSE AND DISCLAIMER
 * ----------------------
 * This material contains sample programming source code ("Sample Code").
 * ScaleOut Software, Inc. (SSI) grants you a nonexclusive license to compile, 
 * link, run, display, reproduce, and prepare derivative works of 
 * this Sample Code.  The Sample Code has not been thoroughly
 * tested under all conditions.  SSI, therefore, does not guarantee
 * or imply its reliability, serviceability, or function. SSI
 * provides no support services for the Sample Code.
 *
 * All Sample Code contained herein is provided to you "AS IS" without
 * any warranties of any kind. THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGMENT ARE EXPRESSLY
 * DISCLAIMED.  SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED
 * WARRANTIES, SO THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO 
 * EVENT WILL SSI BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, 
 * SPECIAL OR OTHER CONSEQUENTIAL DAMAGES FOR ANY USE OF THE SAMPLE CODE
 * INCLUDING, WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS 
 * INTERRUPTION, LOSS OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION
 * HANDLING SYSTEM OR OTHERWISE, EVEN IF WE ARE EXPRESSLY ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGES.
 */

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Diagnostics;
using System.Text;
using System.Threading.Tasks;
using Soss.Client;

namespace Common
{
    [Serializable]
    public class Strategy
    {
        public const string SUMMARY_CACHE_NAME = "Strategy Summary Cache";
        public const string STRATEGY_CACHE_NAME = "Strategies Cache";
        public const string SUMMARY_OBJ_NAME = "Alert Summary";

        static PerformanceCounter _totalUpdatesCounter;
        static PerformanceCounter _updatesPerSecCounter;

        private Collection<Position> _positions;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="id">ID of the new strategy.</param>
        public Strategy(string id)
        {
            _positions = new Collection<Position>();
            this.ID = id;
        }

        /// <summary>
        /// Static constructor. Initializes performance counters.
        /// </summary>
        static Strategy()
        {
            try
            {
                _totalUpdatesCounter = new PerformanceCounter(PerfCounters.CounterCategoryName, PerfCounters.PositionEvalsTotal, false);
                _updatesPerSecCounter = new PerformanceCounter(PerfCounters.CounterCategoryName, PerfCounters.PositionEvalsPerSec, false);
            }
            catch
            {
                // counters probably haven't been installed. Ignore.
            }

        }

        /// <summary>
        /// List of all positions in the strategy.
        /// </summary>
        public Collection<Position> Positions { get { return _positions; } }

        /// <summary>
        /// Indicates whether any core positions exceed the max allowed deviation.
        /// </summary>
        public bool CoreAlert { get; set; }

        /// <summary>
        /// Indicates whether any hedge positions exceed the max allowed deviation.
        /// </summary>
        public bool HedgeAlert { get; set; }

        /// <summary>
        /// ID/Name of the strategy.
        /// </summary>
        public string ID { get; set; }

        public override string ToString()
        {
            return this.ID;
        }

        /// <summary>
        /// Gets the total value of the strategy for the specified position type (hedge or core).
        /// </summary>
        /// <param name="type">hedge or core</param>
        /// <returns>Total strategy value</returns>
        public decimal GetTotalStrategyValue(PositionType type)
        {
            var exp = from p in Positions
                      where p.PositionType == type
                      select p.Exposure;

            return exp.Sum();
        }


        /// <summary>
        /// Updates each position's "ActualAllocationPercent" to reflect current prices and,
        /// if the actual allocation is outside acceptable bounds, sets the the alert status to true.
        /// </summary>
        /// <param name="positionType">hedge or core</param>
        /// <param name="alertThresholdPercent">maximum percentage that a strategy's position is allowed to deviate from the target allocation</param>
        public void UpdateActualAllocations(PositionType positionType, int alertThresholdPercent)
        {
            decimal totalVal = GetTotalStrategyValue(positionType);

            bool alertStrategy = false;
            foreach (Position pos in this.Positions)
            {
                if (pos.PositionType == positionType)
                {
                    pos.ActualAllocationPercent = (pos.Exposure / totalVal) * 100;
                    if (Math.Abs(pos.Deviation) > alertThresholdPercent)
                        pos.Alerted = alertStrategy = true;
                    else
                        pos.Alerted = false;
                }
            }

            if (positionType == PositionType.Core)
                CoreAlert = alertStrategy;
            else
                HedgeAlert = alertStrategy;

        }
        
        /// <summary>
        /// An "Eval" method that is invoked by ScaleOut's Parallel Method Invocation (PMI) engine to update 
        /// prices in a strategy's positions. A NamedCache.Invoke() call can cause this method to be invoked in 
        /// parallel across the farm for every strategy in the distributed datagrid.
        /// </summary>
        /// <param name="strategy">A strategy that has been stored in the distributed datagrid.</param>
        /// <param name="latestPrices">
        /// A PMI "parameter object" that was passed into the NamedCache.Invoke() call, allowing the parameter to be
        /// applied to all positions in the cache. The parameter in this case is a collection of current security prices.
        /// </param>
        /// <param name="evalArgs">
        /// Optional helper argument, much like a .NET EventArgs object, that allows further control of the PMI operation.
        /// </param>
        /// <returns>An AlertSummary collection containing a single item (the name of the strategy and whether it
        /// is alerted). All summaries returned by invocations of this method are merged back into
        /// a single value that is returned to the original NamedCache.Invoke call.</returns>
        public static AlertSummaryCollection UpdatePrices(Strategy strategy, SecurityDictionary latestPrices, EvalArgs evalArgs)
        {
            // Update prices for all positions in the strategy.
            foreach (Position position in strategy.Positions)
            {
                position.Price = latestPrices[position.Ticker].Price;
                if ( _updatesPerSecCounter != null ) _updatesPerSecCounter.Increment();
                if ( _totalUpdatesCounter != null ) _totalUpdatesCounter.Increment();
            }

            strategy.UpdateActualAllocations(PositionType.Core, latestPrices.AlertThresholdPercent);
            strategy.UpdateActualAllocations(PositionType.Hedge, latestPrices.AlertThresholdPercent);

            bool alert = false;
            if (strategy.CoreAlert || strategy.HedgeAlert)
                alert = true;

            AlertSummaryCollection result = new AlertSummaryCollection();
            result.PositionsEvaluated = strategy.Positions.Count;
            if (alert)
                result.Add(strategy.ID);
            

            // set the object disposition to update here so that our strategy
            // updates are persisted back to the StateServer.
            evalArgs.ObjectDisposition = EvalObjectDisposition.Update;

            return result;
        }

        /// <summary>
        /// A "merge" method that is invoked by ScaleOut's Parallel Method Invocation (PMI) engine to combine
        /// results from a distributed operation. In this case, we're just combining alert statuses that were
        /// triggered by a price update -- the collection that is finally returned by the NamedCache.Invoke call will
        /// ultimately contain the alert status for every single strategy in the datagrid.
        /// </summary>
        /// <param name="alertColl1">First set of alerts</param>
        /// <param name="alertColl2">Second set of alerts</param>
        /// <returns>Combined set of alerts.</returns>
        public static AlertSummaryCollection MergeAlerts(AlertSummaryCollection alertColl1, AlertSummaryCollection alertColl2)
        {
            // combine alerts and statistics into a single return value:
            foreach (string item in alertColl2)
            {
                alertColl1.Add(item);
            }

            alertColl1.PositionsEvaluated += alertColl2.PositionsEvaluated;

            return alertColl1;
        }

        
    }
}
