﻿/* 
 * © Copyright 2013 by ScaleOut Software, Inc.
 *
 * LICENSE AND DISCLAIMER
 * ----------------------
 * This material contains sample programming source code ("Sample Code").
 * ScaleOut Software, Inc. (SSI) grants you a nonexclusive license to compile, 
 * link, run, display, reproduce, and prepare derivative works of 
 * this Sample Code.  The Sample Code has not been thoroughly
 * tested under all conditions.  SSI, therefore, does not guarantee
 * or imply its reliability, serviceability, or function. SSI
 * provides no support services for the Sample Code.
 *
 * All Sample Code contained herein is provided to you "AS IS" without
 * any warranties of any kind. THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGMENT ARE EXPRESSLY
 * DISCLAIMED.  SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED
 * WARRANTIES, SO THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO 
 * EVENT WILL SSI BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, 
 * SPECIAL OR OTHER CONSEQUENTIAL DAMAGES FOR ANY USE OF THE SAMPLE CODE
 * INCLUDING, WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS 
 * INTERRUPTION, LOSS OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION
 * HANDLING SYSTEM OR OTHERWISE, EVEN IF WE ARE EXPRESSLY ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGES.
 */

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

using Common;
using Soss.Client;

namespace MarketSimulator
{
    class ExchangeSimulator
    {
        // Maximum % that a simulated price can change:
        private const decimal MAX_PRICE_CHANGE_PERCENT = 1.5M;

        private int _priceUpdatesPerSec;
        private SecurityDictionary _latestPrices;

        private PerformanceCounter _updatesPerSecCounter;
        private PerformanceCounter _totalUpdatesCounter;

        public ExchangeSimulator(SecurityDictionary securities, int priceUpdatesPerSec)
        {
            _priceUpdatesPerSec = priceUpdatesPerSec;
            _latestPrices = new SecurityDictionary();
            foreach (Security security in securities)
                _latestPrices.Add(security.DeepCopy());

            try
            {
                _updatesPerSecCounter = new PerformanceCounter(PerfCounters.CounterCategoryName, PerfCounters.PriceUpdatesPerSec, false);
                _totalUpdatesCounter = new PerformanceCounter(PerfCounters.CounterCategoryName, PerfCounters.PriceUpdatesTotal, false);
            }
            catch
            {
                // counters probably haven't been installed. Ignore.
            }
        }

        public void StartSimulatedFeed()
        {
            Task.Factory.StartNew(new Action(PriceFeedWorker));
        }

        private void PriceFeedWorker()
        {
            Stopwatch stopwatch = new Stopwatch();
            List<string> stockSymbols = _latestPrices.Keys.ToList();
            
            while (true)
            {
                Random rand = new Random();
                stopwatch.Start();

                for (int i = 0; i < _priceUpdatesPerSec; ++i)
                {
                    string symbol = stockSymbols[rand.Next(0, stockSymbols.Count)];

                    decimal oldPrice = _latestPrices[symbol].Price;

                    // Random class only gives us integers. Multiply max allowed by 100, convert rand to decimal, then divide by 100:
                    decimal percentChange = ((decimal)rand.Next(0, (int)(MAX_PRICE_CHANGE_PERCENT * 100) )) / 100.0m;
                    
                    decimal multiplier = percentChange / 100;
                    multiplier = rand.Next(0, 2) == 0 ? multiplier : (multiplier * -1);
                    decimal newPrice = Decimal.Round(oldPrice + (oldPrice * multiplier), 5);

                    lock (_latestPrices)
                    {
                        _latestPrices[symbol].Ticker = symbol;
                        _latestPrices[symbol].Price = newPrice;
                        _latestPrices[symbol].Timestamp = DateTime.Now;
                    }

                    if (_totalUpdatesCounter != null) _totalUpdatesCounter.Increment();
                    if (_updatesPerSecCounter != null) _updatesPerSecCounter.Increment();
                }

                stopwatch.Stop();

                if (stopwatch.Elapsed.Milliseconds <= 1000)
                {
                    // The simulated market feed produced the desired updates/sec with time to spare. Sleep
                    // for the remainder of the 1-second interval. Empirically, it only takes a few milliseconds
                    // to simulate a thousand incoming prices, so this thread spends most of its time in 
                    // this sleep call.
                    int sleepTime = 1000 - stopwatch.Elapsed.Milliseconds;
                    Thread.Sleep(sleepTime);
                }
                else
                {
                    Console.WriteLine("Warning: Simulated market feed was unable to generate {0} prices in one second. Last interval took {1} milliseconds.", _priceUpdatesPerSec, stopwatch.Elapsed.Milliseconds);
                }
                stopwatch.Reset();

            }
        }

        /// <summary>
        /// Returns the latest security prices in this exchange. A deep copy is returned in case multiple threads
        /// are reading/modifying the contents ExchangeSimulator's internal collection.
        /// </summary>
        public SecurityDictionary GetSnapshot()
        {
            SecurityDictionary deepPriceCopy;
            lock (_latestPrices)
            {
                deepPriceCopy = _latestPrices.DeepCopy();
            }
            return deepPriceCopy;
        }

       
    }
}
