﻿/* 
 * © Copyright 2013 by ScaleOut Software, Inc.
 *
 * LICENSE AND DISCLAIMER
 * ----------------------
 * This material contains sample programming source code ("Sample Code").
 * ScaleOut Software, Inc. (SSI) grants you a nonexclusive license to compile, 
 * link, run, display, reproduce, and prepare derivative works of 
 * this Sample Code.  The Sample Code has not been thoroughly
 * tested under all conditions.  SSI, therefore, does not guarantee
 * or imply its reliability, serviceability, or function. SSI
 * provides no support services for the Sample Code.
 *
 * All Sample Code contained herein is provided to you "AS IS" without
 * any warranties of any kind. THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGMENT ARE EXPRESSLY
 * DISCLAIMED.  SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED
 * WARRANTIES, SO THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO 
 * EVENT WILL SSI BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, 
 * SPECIAL OR OTHER CONSEQUENTIAL DAMAGES FOR ANY USE OF THE SAMPLE CODE
 * INCLUDING, WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS 
 * INTERRUPTION, LOSS OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION
 * HANDLING SYSTEM OR OTHERWISE, EVEN IF WE ARE EXPRESSLY ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGES.
 */

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

using Common;
using Soss.Client;

namespace MarketSimulator
{
    class Program
    {
        static void Main(string[] args)
        {
            const int MINIMUM_INTERVAL_BETWEEN_PMI_CALLS = 1000; // in milliseconds
            int strategyCount, priceUpdatesPerSec, positionsPerStrategy;

            try
            {
                if (args.Length == 1)
                {
                    if (args[0].ToLower() == "installcounters")
                    {
                        PerfCounters.InstallCounters();
                        return;
                    }
                    if (args[0].ToLower() == "removecounters")
                    {
                        PerfCounters.UninstallCounters();
                        return;
                    }
                }

                try
                {
                    strategyCount = int.Parse(args[0]);
                }
                catch
                {
                    strategyCount = 2000;
                }
                try
                {
                    priceUpdatesPerSec = int.Parse(args[1]);
                }
                catch
                {
                    priceUpdatesPerSec = 1000;
                }
                try
                {
                    positionsPerStrategy = int.Parse(args[2]);
                }
                catch
                {
                    positionsPerStrategy = 10;
                }

            }
            catch(Exception ex)
            {
                Console.WriteLine("MarketSimulator.exe strategyCount priceUpdatesPerSec positionsPerStrategy " + ex.ToString());
                return;
            }

            Console.WriteLine("{0}: Loading {1} strategies into the data grid, please wait...", DateTime.Now, strategyCount);

            // Generate a list of stock symbols that this market simulator will generate. For the purposes of this simulation,
            // the total symbol count will be half the number of total positions to guarantee that some securities are in multiple strategies.
            int positionCount = strategyCount * positionsPerStrategy;
            int symbolCount = 3000; // rough estimate of securities in NYSE
            SecurityDictionary symbols = GenerateSecurityList(symbolCount);
            if (symbols == null)
            {
                Console.WriteLine("Not enough symbols available.");
                return;
            }

            CreateStrategies(strategyCount, positionsPerStrategy, symbols);

            // Set up an invocation grid. We'll give each client its own unique invocation grid so as to prevent inadvertent
            // reloads (this can happen if you have different builds of the same client running simultaneously).
            InvocationGridBuilder igb = new InvocationGridBuilder("Strategy Worker Grid " + Guid.NewGuid().ToString());

            // Deploy this assembly to all ScaleOut hosts:
            igb.AddDependency(Assembly.GetExecutingAssembly());
            InvocationGrid grid = igb.Load();

            // Start Market Feed thread:
            ExchangeSimulator exchange = new ExchangeSimulator(symbols, priceUpdatesPerSec);
            exchange.StartSimulatedFeed();

            // Perform continuous PMI operations to update prices in positions.
            Stopwatch stopwatch = new Stopwatch();

            NamedCache strategyCache = CacheFactory.GetCache(Strategy.STRATEGY_CACHE_NAME);
            NamedCache strategyAlertSummaryCache = CacheFactory.GetCache(Strategy.SUMMARY_CACHE_NAME);
            NamedCache settingsCache = CacheFactory.GetCache(Settings.SETTINGS_CACHE_NAME);
            strategyCache.InvocationGrid = grid;

            Thread.Sleep(500); // to make sure all strategies have been added before we launch the dashboard app
            // Launching the Strategy Alerts Dashboard program
            LaunchStrategyAlertDesk();

            while (true)
            {
                AlertSummaryCollection alertSummaries = null;
                try
                {
                    // Settings are stored in the grid. We check them with every NamedCache.Invoke call in case there's been a change:
                    Settings settings = settingsCache.Retrieve(Settings.SETTINGS_OBJ_NAME, false) as Settings;
                    if (settings == null)
                    {
                        // Settings aren't currently in the cache. Insert the default:
                        settings = new Settings();
                        settings.MaxAllowedDeviationPercent = Settings.DEFAULT_MAX_DEVIATION_PCT;
                        settingsCache.Insert(Settings.SETTINGS_OBJ_NAME, settings, settingsCache.DefaultCreatePolicy, true, false);
                    }

                    SecurityDictionary latestPrices = exchange.GetSnapshot();
                    latestPrices.AlertThresholdPercent = settings.MaxAllowedDeviationPercent;

                    stopwatch.Restart();
                    alertSummaries = strategyCache.Invoke<Strategy, SecurityDictionary, AlertSummaryCollection>(null, latestPrices, Strategy.UpdatePrices, Strategy.MergeAlerts, TimeSpan.FromMinutes(1));
                    stopwatch.Stop();

                    if (stopwatch.ElapsedMilliseconds < MINIMUM_INTERVAL_BETWEEN_PMI_CALLS)
                    {
                        // Prices in the strategies were updated and evaluated with time to spare. Sleep
                        // for the remainder of the 1-second interval.
                        Thread.Sleep(MINIMUM_INTERVAL_BETWEEN_PMI_CALLS - (int)stopwatch.ElapsedMilliseconds);
                        if ( alertSummaries != null )
                            alertSummaries.EvaluationTime = TimeSpan.FromMilliseconds(MINIMUM_INTERVAL_BETWEEN_PMI_CALLS);
                    }
                    else
                    {
                        if (alertSummaries != null)
                            alertSummaries.EvaluationTime = stopwatch.Elapsed;
                    }

                    Console.WriteLine("{0}: Strategy updates took {1} ms", DateTime.Now, stopwatch.ElapsedMilliseconds);
                    // Store the alert summary in the cache for the dashboard to read:
                    strategyAlertSummaryCache[Strategy.SUMMARY_OBJ_NAME] = alertSummaries;
                }
                catch (MethodInvocationException ex)
                {
                    Console.WriteLine("Distributed invocation failed. Exception(s) follow:");
                    foreach (Exception remoteException in ex.Data[MethodInvocationException.AllExceptionsKey] as IList<Exception>)
                    {
                        Console.WriteLine(remoteException.ToString());
                        Console.WriteLine("-----------");
                    }
                    return;
                }
                catch (Soss.Client.TimeoutException ex)
                {
                    Console.WriteLine(DateTime.Now.ToString() + ": Invocation timeout: " + ex.Message);
                }
                catch (Exception ex)
                {
                    Console.WriteLine("Unexpected exception: " + ex.ToString());
                }
            }

        }

        /// <summary>
        /// Detects if there is already a live instance of the StrategyAlertsDesk.exe process, 
        /// closes it and launches a new instance of it.
        /// </summary>
        public static void LaunchStrategyAlertDesk()
        {
            try
            {
                // Enumerate running processes to see if we have running instances of StrategyAlertsDesk.exe 
                Process[] procs = Process.GetProcesses();
                foreach (Process proc in procs)
                {
                    if (string.Compare(proc.ProcessName, Settings.STRATEGY_ALERTS_DESK_PROCESS, true) == 0)
                    {
                        // If we found one, close the program
                        proc.CloseMainWindow();
                    }
                } // foreach()

                Process process = new Process();
                // Configure the process using the StartInfo properties.
                process.StartInfo.FileName = Path.Combine(Directory.GetCurrentDirectory(), Settings.STRATEGY_ALERTS_DESK_PROCESS + ".exe");
                process.StartInfo.WindowStyle = ProcessWindowStyle.Normal;
                process.Start();
            }
            catch (Exception ex)
            {
                if ( string.Compare(ex.Message, "The system cannot find the file specified", true) == 0 )
                    Console.WriteLine(string.Format("The {0}.exe program was not found in the current directory, please start it manually", Settings.STRATEGY_ALERTS_DESK_PROCESS));
                else
                    Console.WriteLine(string.Format("Unexpected exception occurred while launching the {0}.exe program: ", Settings.STRATEGY_ALERTS_DESK_PROCESS, ex.ToString()));
            }
        }

        /// <summary>
        /// Generates a collection of stock symbols, along with their initial prices.
        /// </summary>
        /// <param name="count">Number of securities to simulate</param>
        /// <returns>A collection of Exchange price objects, which are essentially just Stock/price pairs.</returns>
        public static SecurityDictionary GenerateSecurityList(int count)
        {
            SecurityDictionary securities = new SecurityDictionary();

            Assembly assembly = Assembly.GetExecutingAssembly();
            StreamReader reader = new StreamReader(assembly.GetManifestResourceStream("MarketSimulator.Symbols.txt"));
            Random rand = new Random();

            for (int i = 0; i < count; ++i)
            {
                string symbol = reader.ReadLine();
                if (symbol != null)
                {
                    Security symbolPrice = new Security(symbol, rand.Next(1, 100), DateTime.Now);
                    securities.Add(symbolPrice);
                }
                else
                {
                    // End of file was reached.
                    reader.Close();
                    return null;
                }
            }

            reader.Close();
            return securities;
        }

        /// <summary>
        /// Generates simulated strategy objects and inserts them into the ScaleOut in-memory datagrid.
        /// </summary>
        /// <param name="count">Number of strategies to create.</param>
        /// <param name="positionsPerStrategy">
        /// Number of positions in each strategy. Both a core and hedge position is created for the specified 
        /// value here, so the total positions will be double this parameter.
        /// </param>
        /// <param name="stockSymbols">
        /// A collection of stocks (ExchangePrice objects consisting of ticker symbols and initial prices) that
        /// will be used in the strategies' positions.
        /// </param>
        /// <returns>A list of strategy IDs corresponding to the strategies that were inserted into the in-memory datagrid.</returns>
        static List<string> CreateStrategies(int count, int positionsPerStrategy, SecurityDictionary stockSymbols)
        {
            List<string> strategyKeys = new List<string>(count * positionsPerStrategy);
            decimal percentPerPosition = Decimal.Round((100.0m / positionsPerStrategy), 1);

            NamedCache strategyCache = CacheFactory.GetCache(Strategy.STRATEGY_CACHE_NAME);
            strategyCache.LockRetryInterval = 5;
            CreatePolicy stratPolicy = new CreatePolicy(0); // 0 = infinite lifetime in the datagrid.

            // Bulk insertions like this can be parallelized for improved performance:
            Parallel.For(0, count, strategyId =>
            {
                Strategy strategy = new Strategy("Strategy " + strategyId.ToString("000"));

                Random rand = new Random(strategyId);

                // To initially generate the simulated holdings, we'll first come up with a total core/hedge value for
                // the strategy, and then we'll calculate the number of shares we'd need in each position based on the 
                // price of the security and and percentage it occupies in the strategy. This is a little backwards
                // from reality, but it lets us easily generate strategies with correct numbers and use security prices that
                // are consistent between strategies.
                decimal totalCoreVal = rand.Next(10000, 1000000);
                decimal totalHedgeVal = rand.Next(10000, 1000000);

                for (int positionId = 0; positionId < positionsPerStrategy; positionId++)
                {
                    Security corePosSecurity = stockSymbols[rand.Next(stockSymbols.Count)];
                    Position corePos = new Position()
                    {
                        PositionType = PositionType.Core,
                        Ticker = corePosSecurity.Ticker,
                        TargetAllocationPercent = percentPerPosition,
                        Price = corePosSecurity.Price,
                        Shares = Decimal.Round((totalCoreVal / percentPerPosition / corePosSecurity.Price), 0)
                    };
                    strategy.Positions.Add(corePos);

                    Security hedgePosSecurity = stockSymbols[rand.Next(stockSymbols.Count)];
                    Position hedgePos = new Position()
                    {
                        PositionType = PositionType.Hedge,
                        Ticker = hedgePosSecurity.Ticker,
                        TargetAllocationPercent = percentPerPosition,
                        Price = hedgePosSecurity.Price,
                        Shares = Decimal.Round((totalHedgeVal / percentPerPosition / hedgePosSecurity.Price), 0)
                    };
                    strategy.Positions.Add(hedgePos);
                }

                // Add the strategy to the distributed datagrid:
                strategyCache.Insert(strategy.ID, strategy, stratPolicy, true, false);

                // Lock the returned key collection since we're in a parallel loop.
                lock (strategyKeys)
                {
                    strategyKeys.Add(strategy.ID);
                    
                }
            });

            return strategyKeys;
        }
    }
}
