﻿/* 
 * © Copyright 2013 by ScaleOut Software, Inc.
 *
 * LICENSE AND DISCLAIMER
 * ----------------------
 * This material contains sample programming source code ("Sample Code").
 * ScaleOut Software, Inc. (SSI) grants you a nonexclusive license to compile, 
 * link, run, display, reproduce, and prepare derivative works of 
 * this Sample Code.  The Sample Code has not been thoroughly
 * tested under all conditions.  SSI, therefore, does not guarantee
 * or imply its reliability, serviceability, or function. SSI
 * provides no support services for the Sample Code.
 *
 * All Sample Code contained herein is provided to you "AS IS" without
 * any warranties of any kind. THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGMENT ARE EXPRESSLY
 * DISCLAIMED.  SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED
 * WARRANTIES, SO THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO 
 * EVENT WILL SSI BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, 
 * SPECIAL OR OTHER CONSEQUENTIAL DAMAGES FOR ANY USE OF THE SAMPLE CODE
 * INCLUDING, WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS 
 * INTERRUPTION, LOSS OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION
 * HANDLING SYSTEM OR OTHERWISE, EVEN IF WE ARE EXPRESSLY ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGES.
 */

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using Common;
using Soss.Client;


namespace StrategyAlertsDesk
{
    public partial class Dashboard : Form
    {
        private static decimal nRefreshRate = 2;                    // by default, refresh UI every 2 seconds
        private static string strSelectedStrategyName = string.Empty;
        private static int nDeviationThreshold = 10;
        private static int nNumAlertedPositions = 0;
        private static bool bRetrievingFullListOfStrategies = false;
        private static bool bMonitoring = false;
        private static AutoResetEvent startMonitorEvent = new AutoResetEvent(false);

        private AutoResetEvent statusMonitorWaitHandle = new AutoResetEvent(false);
        private System.ComponentModel.BackgroundWorker bwAlertedStrategy = new BackgroundWorker();
        private static List<string> _strategyFullList = new List<string>();
        private NamedCache strategyCache;
        private NamedCache strategyAlertSummaryCache;
        private NamedCache settingsCache;

        public Dashboard()
        {
            InitializeComponent();

            // Initialize the background worker thread
            InitializeBackgroundWorker();

            Initialize();
        }

        /// <summary>
        /// Initialize class members, reset default UI elements to defaults,
        /// and start monitoring.
        /// </summary>
        void Initialize()
        {
            try
            {
                lblWait.Visible = false;
                spinSec.Value = nRefreshRate;

                // Display check boxes.
                strategyListCtl.CheckBoxes = false;
                // Select the item and subitems when selection is made.
                strategyListCtl.FullRowSelect = true;
                // Display grid lines.
                strategyListCtl.GridLines = true;
                // Sort the items in the list in ascending order.
                strategyListCtl.Sorting = SortOrder.Ascending;

                // Assign the ImageList objects to the ListView.
                strategyListCtl.SmallImageList = imageList1;

                btnStartMon.Enabled = true;
                btnStopMon.Enabled = false;

                // Initialize ScaleOut named caches:
                settingsCache = CacheFactory.GetCache(Settings.SETTINGS_CACHE_NAME);
                strategyCache = CacheFactory.GetCache(Strategy.STRATEGY_CACHE_NAME);
                strategyAlertSummaryCache = CacheFactory.GetCache(Strategy.SUMMARY_CACHE_NAME);

                // Settings are stored in the grid. We check them with every NamedCache.Invoke call in case there's been a change:
                Settings settings = settingsCache.Retrieve(Settings.SETTINGS_OBJ_NAME, false) as Settings;
                if (settings == null)
                {
                    // Settings aren't currently in the cache. Insert the default:
                    settings = new Settings();
                    settings.MaxAllowedDeviationPercent = nDeviationThreshold = Settings.DEFAULT_MAX_DEVIATION_PCT;
                    settingsCache.Insert(Settings.SETTINGS_OBJ_NAME, settings, settingsCache.DefaultCreatePolicy, true, false);
                }
                else
                    nDeviationThreshold = settings.MaxAllowedDeviationPercent;
                txtBoxDevThreshold.Text = Convert.ToString(nDeviationThreshold);

                
            }
            catch (Exception ex)
            {
                MessageBox.Show(string.Format("Error occurred while initializing the Strategy Alerts Dashboard: {0}", ex.Message));
            }
        }
        
        /// <summary>
        /// Get sorted list of all strategies stored in the corresponding named cache
        /// in a dedicated thread using .NET's Task Parallel Library
        /// </summary>
        void GetListOfStrategies()
        {
            NamedCache strategyNC = CacheFactory.GetCache(Strategy.STRATEGY_CACHE_NAME);
            int nCount = strategyNC.Count;

            if (strategyNC.Count != _strategyFullList.Count && !bRetrievingFullListOfStrategies)
            {
                bRetrievingFullListOfStrategies = true;

                lblWait.Visible = true;
                _strategyFullList.Clear();

                // Starting a new worker thread inline here to get a list of all strategies
                (new Thread(() =>
                {
                    // Calculate progress bar update increment
                    int nInc = (int) nCount / 50;
                    int nIdx = 0, nProgress = 1, nStep = 2;
                    // Get keys for all Strategies stored in the ScaleOut IMDG:
                    var keyCol = strategyNC.Keys;

                    lock (_strategyFullList)
                    {
                        // Populate the list using parallel loop since the GetStringKey() call
                        // does the round trip to the server:
                        Parallel.ForEach(keyCol.Cast<CachedObjectId>(), keyElem =>
                        {
                            if ( (nIdx++) % nInc == 0 )
                            {
                                LoadProgressBar.BeginInvoke(new Action(() => LoadProgressBar.Value = nProgress), null);
                                nProgress += nStep;
                                if (nProgress > 100)
                                    nProgress = 100;
                            }

                            _strategyFullList.Add(keyElem.GetStringKey());
                        });

                        // Sort the list:
                        IList<string> list = new List<string>(_strategyFullList);
                        IEnumerable<string> sortedEnum = list.OrderBy(s => s);
                        _strategyFullList = sortedEnum.ToList();
                    }
                    bRetrievingFullListOfStrategies = false;
                    startMonitorEvent.Set();

                    // Update UI:
                    lblWait.BeginInvoke(new Action(() => lblWait.Visible = false), null);
                    LoadProgressBar.BeginInvoke(new Action(() => LoadProgressBar.Value = 0), null);
                })).Start();
            }
        }

        /// <summary>
        /// Gets a list of alerted strategies
        /// </summary>
        /// <returns>Instance of the AlertSummaryCollection class stored in ScaleOut StateServer</returns>
        private AlertSummaryCollection GetAlertedStrategies()
        {
            AlertSummaryCollection strategyList = strategyAlertSummaryCache.Get(Strategy.SUMMARY_OBJ_NAME) as AlertSummaryCollection;
            if ( strategyList != null )
                nNumAlertedPositions = strategyList.Count;

            return strategyList;
        }

        /// <summary>
        /// Pulls the strategy summary object from the store
        /// </summary>
        /// <param name="strStrategyName">Strategy name</param>
        /// <returns>instance of the Strategy class</returns>
        Strategy GetStrategyDetails(string strStrategyName)
        {
            Strategy strategyObj = null;

            if (!string.IsNullOrEmpty(strStrategyName))
            {
                strategyObj = strategyCache.Get(strStrategyName) as Strategy;
            }

            return strategyObj;
        }

        /// <summary>
        /// Updates the list of strategies in ListView control and marks them as alerted. If
        /// a strategy is selected, that strategy's details are updated in the grid control.
        /// </summary>
        /// <param name="objStarategyState">instance of the StrategyState class</param>
        private void UpdateUIState(StrategyState objStarategyState)
        {
            if (objStarategyState != null)
            {
                UpdateStrategyList(objStarategyState.strategyList);

                UpdateStrategyDetailsGrid(objStarategyState.SelectedStrategy);

                UpdateAggregateParameters();
            }
        }

        /// <summary>
        /// Updates the strategy list control with the latest list of strategies.
        /// By default we only display the first 100 strategies if their total number
        /// is greater than 100.
        /// </summary>
        /// <param name="strategyList">instance of the AlertSummaryCollection class</param>
        private void UpdateStrategyList(AlertSummaryCollection strategyList)
        {
            try
            {
                bool bAlerted = false;
                bool bMoreItemsAvailable = false;
                int nDisplayedNumber = 0, nCount = 0, nIdx = 0;

                nDisplayedNumber = nCount = _strategyFullList.Count;

                if (nDisplayedNumber > Settings.NUMBER_OF_DYSPLAYED_STRATEGIES)
                {
                    nDisplayedNumber = Settings.NUMBER_OF_DYSPLAYED_STRATEGIES;
                    bMoreItemsAvailable = true;
                }


                lock (_strategyFullList)
                {
                    // Update the list 
                    foreach (string strStrategyName in _strategyFullList)
                    {
                        if (nIdx++ >= nDisplayedNumber) // only display the first 100
                            break;

                        if (strategyList != null && strategyList.Contains(strStrategyName))
                            bAlerted = true;
                        else
                            bAlerted = false;

                        // Update or add item in the UI list of strategies:
                        UpdateLineItem(strStrategyName, bAlerted);
                    } // foreach
                }

                if ( bMoreItemsAvailable )
                    lblStrategyList.Text = string.Format("Strategy List (first {0} items out of {1}):", Settings.NUMBER_OF_DYSPLAYED_STRATEGIES, nCount);
                else
                    lblStrategyList.Text = "Strategy List:";
            }
            catch (Exception ex)
            {
                MessageBox.Show(string.Format("Error occurred while updating the strategy list in the form: {0}", ex.Message));
            }
        }

        /// <summary>
        /// Sets the background color of the line item in the list of Strategies depending on whether the 
        /// corresponing strategy is alerted or not. If strategy is not in the list
        /// control, it adds it as a new item
        /// </summary>
        /// <param name="strStrategyName">strategy name</param>
        /// <param name="bAlerted">whether this stategy is alerted or not</param>
        private void UpdateLineItem(string strStrategyName, bool bAlerted)
        {
            ListViewItem lvItem;

            if (!string.IsNullOrEmpty(strStrategyName))
            {
                lvItem = strategyListCtl.Items[strStrategyName];
                if (lvItem != null)
                {
                    if (bAlerted == true)
                    {
                        lvItem.BackColor = Color.LemonChiffon;
                        lvItem.ImageIndex = 1;
                    }
                    else
                    {
                        lvItem.BackColor = Color.White;
                        lvItem.ImageIndex = 0;
                    }
                }
                else
                {
                    if (strategyListCtl.Items.Count < Settings.NUMBER_OF_DYSPLAYED_STRATEGIES)
                    {
                        // Create a new strategy item
                        ListViewItem lvNewItem = new ListViewItem(strStrategyName);
                        lvNewItem.Name = strStrategyName;
                        if (bAlerted == true)
                        {
                            lvNewItem.BackColor = Color.LemonChiffon;
                            lvNewItem.ImageIndex = 1;
                        }
                        else
                        {
                            lvNewItem.BackColor = Color.White;
                            lvNewItem.ImageIndex = 0;
                        }
                        strategyListCtl.Items.Add(lvNewItem);
                    }
                }
            }
        }

        /// <summary>
        /// Updates the data grid control with the latest strategy details
        /// </summary>
        /// <param name="objStrategy">instance of the Strategy class</param>
        private void UpdateStrategyDetailsGrid(Strategy objStrategy)
        {
            try
            {
                if (objStrategy != null)
                {
                    IList<Position> list = new List<Position>(objStrategy.Positions);
                    IEnumerable<Position> sortedEnum = list.OrderBy(f => f.PositionType);
                    IList<Position> sortedList = sortedEnum.ToList();

                    StrategyDetails.DataSource = sortedList;
                    // Formating the data grid
                    FormatDataGrid();
                }
                else
                {
                    lblStrategyDetails.Text = "Strategy details:";
                    StrategyDetails.DataSource = null;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(string.Format("Error occurred while updating the strategy details on the form: {0}", ex.Message));
            }
        }

        /// <summary>
        /// Updates the form with 2 aggregated parameters calculated by the MarketSimulator program
        /// </summary>
        private void UpdateAggregateParameters()
        {
            AlertSummaryCollection alertSummaries = strategyAlertSummaryCache.Get(Strategy.SUMMARY_OBJ_NAME) as AlertSummaryCollection;
            if (alertSummaries != null)
            {
                lblPosEval.Text = (alertSummaries.PositionsEvaluated > 0) ? string.Format("{0:N0}", alertSummaries.PositionsEvaluated) : "-";
                if (alertSummaries.EvaluationTime >= TimeSpan.Zero)
                {
                    double throughput = (double)alertSummaries.PositionsEvaluated / alertSummaries.EvaluationTime.TotalSeconds;
                    lblThroughput.Text = string.Format("{0:N0}", throughput);
                }
                else
                    lblThroughput.Text = "-";

                lblNumAlertedStrategies.Text = (nNumAlertedPositions > 0) ? string.Format("{0:N0}", nNumAlertedPositions) : "0";
            }
        }

        /// <summary>
        /// Starts the monitoring background worker thread
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnStartMon_Click(object sender, EventArgs e)
        {
            btnStartMon.Enabled = false;
            btnStopMon.Enabled = true;
            BackgroundWorkerParameters prm = new BackgroundWorkerParameters();
            prm.statusMonitorWaitHandle = statusMonitorWaitHandle;

            if ( !bMonitoring )
                // Start the background worker
                bwAlertedStrategy.RunWorkerAsync(prm);

            bMonitoring = true;
        }

        /// <summary>
        /// Stops monitoring worker thread
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnStopMon_Click(object sender, EventArgs e)
        {
            btnStartMon.Enabled = true;
            btnStopMon.Enabled = false;

            // Stop the worker
            statusMonitorWaitHandle.Set();
            bMonitoring = false;
        }

        /// <summary>
        /// Updates detail grid to display positions of the currently selected strategy
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void strategyListCtl_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                ListView.SelectedListViewItemCollection selStrategy = this.strategyListCtl.SelectedItems;
                foreach (ListViewItem item in selStrategy)
                {
                    Dashboard.strSelectedStrategyName = item.Text;
                    break; // only support one selection
                }
                if (!string.IsNullOrEmpty(Dashboard.strSelectedStrategyName))
                {
                    lblStrategyDetails.Text = string.Format("{0} details:", strSelectedStrategyName);

                    Strategy objStrategy = GetStrategyDetails(Dashboard.strSelectedStrategyName);
                    UpdateStrategyDetailsGrid(objStrategy);
                }
                else
                {
                    lblStrategyDetails.Text = "Strategy details:";
                    StrategyDetails.DataSource = null;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(string.Format("Error occurred by selecting the strategy item: {0}", ex.Message));
            }
        }

        /// <summary>
        /// Applies formating and cell alignment on the data grid contol
        /// </summary>
        private void FormatDataGrid()
        {
            try
            {
                StrategyDetails.ColumnHeadersDefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter;
                
                StrategyDetails.Columns["Exposure"].Visible = true;

                // Formating, alignment
                StrategyDetails.Columns["TargetAllocationPercent"].DefaultCellStyle.Format = "#.00\\%";
                StrategyDetails.Columns["TargetAllocationPercent"].DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight;
                StrategyDetails.Columns["Price"].DefaultCellStyle.Format = "C2";
                StrategyDetails.Columns["Price"].DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight;
                StrategyDetails.Columns["Shares"].DefaultCellStyle.Format = "N0";
                StrategyDetails.Columns["Shares"].DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight;
                StrategyDetails.Columns["ActualAllocationPercent"].DefaultCellStyle.Format = "#.00\\%";
                StrategyDetails.Columns["ActualAllocationPercent"].DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight;
                StrategyDetails.Columns["Deviation"].DefaultCellStyle.Format = "#.00\\%";
                StrategyDetails.Columns["Deviation"].DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight;
                StrategyDetails.Columns["Exposure"].DefaultCellStyle.Format = "C2";
                StrategyDetails.Columns["Exposure"].DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight;

                // Highlight rows for alerted securities
                foreach (DataGridViewRow raw in StrategyDetails.Rows)
                {
                    if ( Convert.ToBoolean(raw.Cells["Alerted"].Value) == true )
                        raw.DefaultCellStyle.BackColor = Color.LemonChiffon;
                }
            }
            catch (FormatException exf)
            {
                string strExtra = string.Empty;
                if (exf.Data != null)
                {
                    strExtra = " Additional information:";
                    foreach (DictionaryEntry de in exf.Data)
                        strExtra += string.Format(" key:'{0}'; value is: {1}\n", de.Key, de.Value);
                }

                MessageBox.Show(string.Format("Error occurred applying formating to the data grid control: {0}{1}", exf.Message, strExtra));
            }
            catch (Exception ex)
            {
                MessageBox.Show(string.Format("Error occurred applying formatting to the data grid control: {0}", ex.Message));
            }
        }

        #region Worker thread to pull the strategy specific information
        /// <summary>
        /// Set up the BackgroundWorker event handlers. 
        /// </summary>
        private void InitializeBackgroundWorker()
        {
            bwAlertedStrategy.WorkerReportsProgress = true;
            bwAlertedStrategy.WorkerSupportsCancellation = true;

            bwAlertedStrategy.DoWork += new DoWorkEventHandler(bwAS_DoWork);
            bwAlertedStrategy.RunWorkerCompleted += new RunWorkerCompletedEventHandler(bwAS_RunWorkerCompleted);
            bwAlertedStrategy.ProgressChanged += new ProgressChangedEventHandler(bwAS_ProgressChanged);
        }

        /// <summary>
        /// Event handler of the worker thread
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void bwAS_DoWork(object sender, DoWorkEventArgs e)
        {
            // Get the BackgroundWorker that raised this event.
            BackgroundWorker worker = sender as BackgroundWorker;

            // Assign the result of the call to the Result property 
            // of the DoWorkEventArgs object. 
            e.Result = DoMonitoring(e.Argument, worker, e);
        }

        /// <summary>
        /// Result handler for the background worker.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void bwAS_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            string strErrorMessage = string.Empty;
            StrategyState objStarategyState = null;

            if (e.Error != null)
            {
                strErrorMessage = string.Format("{0}. {1}", e.Error.Message, (e.Error.InnerException != null) ?
                                                e.Error.InnerException.Message : string.Empty);
                MessageBox.Show(strErrorMessage);
            }
            else
            {
                try
                {
                    // The operation successfully finished.
                    objStarategyState = e.Result as StrategyState;

                    UpdateUIState(objStarategyState);
                }
                catch (Exception ex)
                {
                    strErrorMessage = string.Format("Error occurred while updating the strategy list: {0} {1}",
                                                    ex.Message, (ex.InnerException != null) ? ex.InnerException.Message : string.Empty);
                    MessageBox.Show(strErrorMessage);
                }
            }
        }

        /// <summary>
        /// Handler that receives periodic progress updates from the background worker thread
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void bwAS_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            StrategyState objStarategyState = null;
            string strErrorMessage = string.Empty;

            try
            {
                objStarategyState = e.UserState as StrategyState;
                if (objStarategyState != null)
                {
                    UpdateUIState(objStarategyState);
                }
            }
            catch (Exception ex)
            {
                strErrorMessage = string.Format("Error occurred while updating the strategy list: {0} {1}", ex.Message, (ex.InnerException != null) ? ex.InnerException.Message : string.Empty);
                MessageBox.Show(strErrorMessage);
            }
        }

        /// <summary>
        /// A worker method that is run asynchronously and retrieves a list of strategies.  
        /// If one of them was selected by user, it retrieves details about it.
        /// </summary>
        /// <param name="param">not used</param>
        /// <param name="worker">live instance of the BackgroundWorker class</param>
        /// <param name="e">instance of the DoWorkEventArgs class</param>
        /// <returns>instance of the StrategyState class</returns>
        private StrategyState DoMonitoring(object objParams, BackgroundWorker worker, DoWorkEventArgs e)
        {
            Strategy objStrategy = null;
            AlertSummaryCollection strategyList = null;
            StrategyState objStarategyState = new StrategyState();

            // If we're retrieving a full set of strategies, just wait until it's finished
            if ( bRetrievingFullListOfStrategies )
                startMonitorEvent.WaitOne();

            while (!(objParams as BackgroundWorkerParameters).statusMonitorWaitHandle.WaitOne((int)nRefreshRate*1000))
            {
                try
                {
                    // First, get the list of alerted strategies:
                    strategyList = GetAlertedStrategies();

                    // Get detailed infoprmation about selected strategy
                    objStrategy = GetStrategyDetails(Dashboard.strSelectedStrategyName);

                    objStarategyState.SelectedStrategy = objStrategy;
                    objStarategyState.strategyList = strategyList;
                }
                catch (Exception ex)
                {
                    throw new ApplicationException("Error occurred while getting the summary list of strategies", ex);
                }

                worker.ReportProgress(100, objStarategyState);
            }

            return objStarategyState;
        }
        #endregion

        /// <summary>
        /// Event handler for spin control
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void spinSec_ValueChanged(object sender, EventArgs e)
        {
            nRefreshRate = spinSec.Value;
        }

        /// <summary>
        /// Event handler for the alert threshold text box
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void txtBoxDevThreshold_TextChanged(object sender, EventArgs e)
        {
            try
            {
                if (!string.IsNullOrEmpty(txtBoxDevThreshold.Text)) // No threshold applied - the last threshold value will be used
                {
                    nDeviationThreshold = Convert.ToInt32(txtBoxDevThreshold.Text);

                    Settings settings = settingsCache.Retrieve(Settings.SETTINGS_OBJ_NAME, false) as Settings;
                    if (settings == null)
                        // Settings aren't currently in the cache. Insert the default:
                        settings = new Settings();

                    settings.MaxAllowedDeviationPercent = nDeviationThreshold;
                    settingsCache.Insert(Settings.SETTINGS_OBJ_NAME, settings, settingsCache.DefaultCreatePolicy, true, false);
                }

            }
            catch (FormatException exf)
            {
                MessageBox.Show(string.Format("The deviation threshold must be a percent value from 0 to 100. {0}", exf.Message));
            }
        }

        private void Dashboard_Load(object sender, EventArgs e)
        {
            try
            {
                // Retrieve full list of strategies
                GetListOfStrategies();

                // Populate the List control
                UpdateStrategyList(null);

                // Start monitoring process
                btnStartMon_Click(null, null);
            }
            catch (Exception ex)
            {
                throw new ApplicationException("Error occurred while getting the summary list of strategies", ex);
            }
        }
    }

    /// <summary>
    /// An instance of this class is passed to the background worker as an argument
    /// </summary>
    class BackgroundWorkerParameters
    {
        // Event handle that is signaled when the monitoring needs to be stopped
        public AutoResetEvent statusMonitorWaitHandle;
    }

    /// <summary>
    /// Internal state class to share between UI and worker thread
    /// </summary>
    internal class StrategyState
    {
        public Strategy SelectedStrategy { get; set; }
        public AlertSummaryCollection strategyList { get; set; }
    }
}
